@@ -113,61 +113,80 @@ get_utf16_string (void* buffer, gint length, guint byte_order)
 	
 }
 
-static char *
-mhod3_get_ithmb_filename (DBParseContext *ctx, ArtworkDB_MhodHeaderArtworkType3 *mhod3)
+struct ParsedMhodString {
+        enum MhodType mhod_type;
+        char *mhod_string;
+};
+
+static struct ParsedMhodString *
+parse_mhod_string (DBParseContext *ctx, GError *error)
 {
-       char *filename=NULL;
+	struct ParsedMhodString *result;
+	ArtworkDB_MhodHeaderString *mhod_string;
+	ArtworkDB_MhodHeader *mhod;
+	gint len;
+	mhod = db_parse_context_get_m_header (ctx, ArtworkDB_MhodHeader, "mhod");
+	if (mhod == NULL) {
+		return NULL;
+	}
+	db_parse_context_set_total_len (ctx, get_gint32 (mhod->total_len, ctx->byte_order));
 
-       g_assert (mhod3 != NULL);
+	if (get_gint32 (mhod->total_len, ctx->byte_order) < sizeof (ArtworkDB_MhodHeaderString)){
+		return NULL;
+	}
 
-       if (mhod3->mhod_version == 2)
-	   filename = get_utf16_string ((gunichar2 *)mhod3->string,
-					get_gint32 (mhod3->string_len, ctx->byte_order),
-					ctx->byte_order);
-       else if ((mhod3->mhod_version == 0) ||
-		(mhod3->mhod_version == 1))
-	   filename = g_strndup (mhod3->string,
-				 get_gint32 (mhod3->string_len, ctx->byte_order));
-       else
-	   g_warning (_("Unexpected mhod3 string type: %d\n"),
-		      mhod3->mhod_version);
-       return filename;
+	result = g_new0 (struct ParsedMhodString, 1);
+	if (result == NULL) {
+		return NULL;
+	}
+
+	mhod_string = (ArtworkDB_MhodHeaderString*)mhod;
+	result->mhod_type = get_gint16 (mhod_string->type, ctx->byte_order);
+	len = get_gint32 (mhod_string->string_len, ctx->byte_order);
+	switch (mhod_string->encoding) {
+	case 2:
+		result->mhod_string = get_utf16_string ((gunichar2 *)mhod_string->string,
+							len, ctx->byte_order);
+		break;
+	case 0:
+	case 1: 
+		result->mhod_string = g_strndup (mhod_string->string, len);
+		break;
+	default:
+		g_warning (_("Unexpected mhod string type: %d\n"),
+			   mhod_string->encoding);
+		break;
+	}
+	dump_mhod_string (mhod_string);
+	return result;
 }
 
-
 static int
 parse_mhod_3 (DBParseContext *ctx,
-	      Itdb_Thumb *thumb, GError *error)
+	      Itdb_Thumb_Ipod_Item *thumb, GError *error)
 {
-	ArtworkDB_MhodHeader *mhod;
-	ArtworkDB_MhodHeaderArtworkType3 *mhod3;
-	gint32 mhod3_type;
-
-	mhod = db_parse_context_get_m_header (ctx, ArtworkDB_MhodHeader, "mhod");
+	struct ParsedMhodString *mhod;
+	mhod = parse_mhod_string (ctx, error);
 	if (mhod == NULL) {
 		return -1;
 	}
-	db_parse_context_set_total_len (ctx, get_gint32 (mhod->total_len, ctx->byte_order));
-	
-	if (get_gint32 (mhod->total_len, ctx->byte_order) < sizeof (ArtworkDB_MhodHeaderArtworkType3)){
+	if (mhod->mhod_type != MHOD_ARTWORK_TYPE_FILE_NAME) {
+		g_free (mhod->mhod_string);
+		g_free (mhod);
 		return -1;
 	}
-	mhod3 = (ArtworkDB_MhodHeaderArtworkType3*)mhod;
-	mhod3_type = get_gint16 (mhod3->type, ctx->byte_order);
-	if (mhod3_type != MHOD_ARTWORK_TYPE_FILE_NAME) {
-		return -1;
-	}
-	thumb->filename = mhod3_get_ithmb_filename (ctx, mhod3);
-	dump_mhod_type_3 (mhod3);
+
+	thumb->filename = mhod->mhod_string;
+	g_free (mhod);
 	return 0;
 }
 
 static int
-parse_photo_mhni (DBParseContext *ctx, Itdb_Artwork *artwork, GError *error)
+parse_photo_mhni (DBParseContext *ctx, Itdb_Thumb_Ipod *thumbs, GError *error)
 {
 	MhniHeader *mhni;
 	DBParseContext *mhod_ctx; 
-	Itdb_Thumb *thumb;
+	Itdb_Thumb_Ipod_Item *thumb;
 
 	mhni = db_parse_context_get_m_header (ctx, MhniHeader, "mhni");
 	if (mhni == NULL) {
@@ -180,8 +199,8 @@ parse_photo_mhni (DBParseContext *ctx, Itdb_Artwork *artwork, GError *error)
 	if (thumb == NULL) {
 		return 0;
 	}
-
-	artwork->thumbnails = g_list_append (artwork->thumbnails, thumb);
+    
+        itdb_thumb_ipod_add (thumbs, thumb);
 
 	mhod_ctx = db_parse_context_get_sub_context (ctx, ctx->header_len);
 	if (mhod_ctx == NULL) {
@@ -194,7 +213,7 @@ parse_photo_mhni (DBParseContext *ctx, Itdb_Artwork *artwork, GError *error)
 }
 
 static int
-parse_photo_mhod (DBParseContext *ctx, Itdb_Artwork *artwork, GError *error)
+parse_photo_mhod (DBParseContext *ctx, Itdb_Thumb_Ipod *thumbs, GError *error)
 {
 	ArtworkDB_MhodHeader *mhod;
 	DBParseContext *mhni_ctx;
@@ -208,11 +227,7 @@ parse_photo_mhod (DBParseContext *ctx, Itdb_Artwork *artwork, GError *error)
 
 	type = get_gint16 (mhod->type, ctx->byte_order);
 
-	if ( type == MHOD_ARTWORK_TYPE_ALBUM_NAME) {
-		dump_mhod_type_1 ((MhodHeaderArtworkType1 *)mhod);
-	} else {
-		dump_mhod (mhod);
-	}
+        dump_mhod (mhod);
 
 	/* if this is a container... */
 	if (type == MHOD_ARTWORK_TYPE_THUMBNAIL) {
@@ -220,12 +235,13 @@ parse_photo_mhod (DBParseContext *ctx, Itdb_Artwork *artwork, GError *error)
 		if (mhni_ctx == NULL) {
 			return -1;
 		}
-		parse_photo_mhni (mhni_ctx, artwork, NULL);
+		parse_photo_mhni (mhni_ctx, thumbs, NULL);
 		g_free (mhni_ctx);
 	}
 
 	return 0;
 }
+
 static int
 parse_mhii (DBParseContext *ctx, GError *error)
 {
@@ -233,14 +249,12 @@ parse_mhii (DBParseContext *ctx, GError *error)
 	DBParseContext *mhod_ctx;
 	int num_children;
 	off_t cur_offset;
-	iPodSong *song = NULL;
 	Itdb_Artwork *artwork;
 	Itdb_PhotoDB *photodb;
-	Itdb_iTunesDB *itunesdb;
-	guint64 dbid;
 	guint64 mactime;
 	Itdb_Device *device = db_get_device (ctx->db);
-
+        Itdb_Thumb_Ipod *thumbs;
+            
 	mhii = db_parse_context_get_m_header (ctx, MhiiHeader, "mhii");
 	if (mhii == NULL)
 	{
@@ -249,32 +263,7 @@ parse_mhii (DBParseContext *ctx, GError *error)
 	db_parse_context_set_total_len (ctx, get_gint32 (mhii->total_len, ctx->byte_order));
 	dump_mhii (mhii);
 
-	switch (ctx->db->db_type)
-	{
-	case DB_TYPE_PHOTO:
-	    photodb = db_get_photodb (ctx->db);
-	    g_return_val_if_fail (photodb, -1);
-	    artwork = g_new0 (Itdb_Artwork, 1);
-	    photodb->photos = g_list_append (photodb->photos, artwork);
-	    break;
-	case DB_TYPE_ITUNES:
-	    itunesdb = db_get_itunesdb (ctx->db);
-	    g_return_val_if_fail (itunesdb, -1);
-	    dbid = get_gint64 (mhii->song_id, ctx->byte_order);
-	    song = get_song_by_dbid (itunesdb, dbid);
-	    if (song == NULL)
-	    {
-		gchar *strval = g_strdup_printf("%" G_GINT64_FORMAT, dbid);
-		g_print (_("Could not find corresponding track (dbid: %s) for artwork entry.\n"), strval);
-		g_free (strval);
-		return -1;
-	    }
-	    artwork = song->artwork;
-	    g_return_val_if_fail (artwork, -1);
-	    break;
-	default:
-	    g_return_val_if_reached (-1);
-	}
+	artwork = itdb_artwork_new ();
 
 	artwork->id = get_gint32 (mhii->image_id, ctx->byte_order);
 	artwork->unk028 = get_gint32 (mhii->unknown4, ctx->byte_order);
@@ -285,30 +274,38 @@ parse_mhii (DBParseContext *ctx, GError *error)
 	mactime = get_gint32 (mhii->digitized_date, ctx->byte_order);
 	artwork->digitized_date = device_time_mac_to_time_t (device, mactime);
 	artwork->artwork_size = get_gint32 (mhii->orig_img_size, ctx->byte_order);
+	artwork->dbid = get_gint64 (mhii->song_id, ctx->byte_order);
 
-	if (song)
-	{
-	    if ((song->artwork_size+song->artwork_count) !=
-		artwork->artwork_size)
-	    {
-		g_warning (_("iTunesDB and ArtworkDB artwork sizes inconsistent (%d+%d != %d)\n"),
-			   song->artwork_size,
-			   song->artwork_count,
-			   song->artwork->artwork_size);
-	    }
-	}
-
+        thumbs = (Itdb_Thumb_Ipod *)itdb_thumb_ipod_new ();
+        artwork->thumbnail = (Itdb_Thumb *)thumbs;
 	cur_offset = ctx->header_len;
 	mhod_ctx = db_parse_context_get_sub_context (ctx, cur_offset);
 	num_children = get_gint32 (mhii->num_children, ctx->byte_order);
 	while ((num_children > 0) && (mhod_ctx != NULL))
 	{
-	    parse_photo_mhod (mhod_ctx, artwork, NULL);
+	    parse_photo_mhod (mhod_ctx, thumbs, NULL);
 	    num_children--;
 	    cur_offset += mhod_ctx->total_len;
 	    g_free (mhod_ctx);
 	    mhod_ctx = db_parse_context_get_sub_context (ctx, cur_offset);
 	}
+        g_free (mhod_ctx);
+
+	switch (ctx->db->db_type)
+	{
+	case DB_TYPE_PHOTO:
+	    photodb = db_get_photodb (ctx->db);
+	    g_return_val_if_fail (photodb, -1);
+	    photodb->photos = g_list_append (photodb->photos, artwork);
+	    break;
+	case DB_TYPE_ITUNES:
+	    g_return_val_if_fail (ctx->artwork!=NULL, -1);
+	    *ctx->artwork = g_list_prepend (*ctx->artwork, artwork);
+	    break;
+	default:
+	    g_return_val_if_reached (-1);
+	}
+
 	return 0;
 }
 
@@ -316,6 +313,7 @@ static int
 parse_mhba (DBParseContext *ctx, GError *error)
 {
 	MhbaHeader *mhba;
+	DBParseContext *mhod_ctx;
 	DBParseContext *mhia_ctx;
 	Itdb_PhotoAlbum *album;
 	Itdb_PhotoDB *photodb;
@@ -352,37 +350,35 @@ parse_mhba (DBParseContext *ctx, GError *error)
 
 	cur_offset = ctx->header_len;
 	num_children = get_gint32 (mhba->num_mhods, ctx->byte_order);
-	while (num_children > 0)
-	{
-	    MhodHeaderArtworkType1 *mhod1;
-	    ArtworkDB_MhodHeader *mhod;
-	    DBParseContext *mhod_ctx;
 
-	    mhod_ctx = db_parse_context_get_sub_context (ctx, cur_offset);
-	    /* FIXME: First mhod is album name, whats the others for? */
-	    mhod = db_parse_context_get_m_header (mhod_ctx,
-						  ArtworkDB_MhodHeader, "mhod");
-	    if (mhod == NULL) {
+        mhod_ctx = db_parse_context_get_sub_context (ctx, cur_offset);
+        while ((num_children > 0) && (mhod_ctx != NULL)) {
+		struct ParsedMhodString *mhod;
+
+		mhod = parse_mhod_string (mhod_ctx, error);
+		if (mhod == NULL) {
+			break;
+		}
+		switch (mhod->mhod_type)
+		{  /* FIXME: type==1 is album name. type==2 seems to be
+		    * the transtition type between photos,
+		    * e.g. "Dissolve". Not handled yet. */
+		case MHOD_ARTWORK_TYPE_ALBUM_NAME:
+			g_free (album->name);
+			album->name = mhod->mhod_string;
+			g_free (mhod);
+			break;
+		default:
+			g_free (mhod->mhod_string);
+			g_free (mhod);
+			break;
+		}
+		cur_offset += mhod_ctx->total_len;
 		g_free (mhod_ctx);
-		return -1;
-	    }
-	    db_parse_context_set_total_len (mhod_ctx,
-					    get_gint32(mhod->total_len, ctx->byte_order));
-	    mhod1 = (MhodHeaderArtworkType1*)mhod;
-	    switch (mhod1->type)
-	    {  /* FIXME: type==1 is album name. type==2 seems to be
-		* the transtition type between photos,
-		* e.g. "Dissolve". Not handled yet. */
-	    case 1:
-		album->name = g_strndup ((gchar *)mhod1->string,
-					 get_gint32(mhod1->string_len, ctx->byte_order));
-		dump_mhod_type_1 (mhod1);
-		break;
-	    }
-	    cur_offset += mhod_ctx->total_len;
-	    g_free (mhod_ctx);
-	    num_children--;
-	}
+		num_children--;
+		mhod_ctx = db_parse_context_get_sub_context (ctx, cur_offset);
+        }
+	g_free (mhod_ctx);
 
 	mhia_ctx = db_parse_context_get_sub_context (ctx, cur_offset);
 	num_children = get_gint32 (mhba->num_mhias, ctx->byte_order);
@@ -393,8 +389,10 @@ parse_mhba (DBParseContext *ctx, GError *error)
 		g_free (mhia_ctx);
 		mhia_ctx = db_parse_context_get_sub_context (ctx, cur_offset);
 	}
+        g_free (mhia_ctx);
 	photodb = db_get_photodb (ctx->db);
 	g_return_val_if_fail (photodb, -1);
+	album->photodb = photodb;
 	photodb->photoalbums = g_list_append (photodb->photoalbums,
 					      album);
 	return 0;
@@ -433,6 +431,7 @@ parse_mhl (DBParseContext *ctx, GError *error,
 		g_free (mhi_ctx);
 		mhi_ctx = db_parse_context_get_sub_context (ctx, cur_offset);
 	}
+        g_free (mhi_ctx);
 
 	return 0;
 
@@ -483,6 +482,110 @@ parse_mhsd (DBParseContext *ctx, GError **error)
 	return 0;
 }
 
+
+/* Apple introduced a new way to associate artwork. The former way
+ * used the dbid to link each artwork (mhii) back to the track. The
+ * new way uses the mhii id to link from each track to the mhii. Above
+ * we only handled the former way */
+static int
+mhfd_associate_itunesdb_artwork (DBParseContext *ctx)
+{
+    GHashTable *mhii_id_hash;
+    Itdb_iTunesDB *itdb;
+    GList *gl;
+
+    g_return_val_if_fail (ctx && ctx->artwork, -1);
+    itdb = db_get_itunesdb (ctx->db);
+    g_return_val_if_fail (itdb, -1);
+
+    /* make a hash linking the mhii with the artwork for faster
+       lookup */
+    mhii_id_hash = g_hash_table_new_full (g_direct_hash,
+					  g_direct_equal,
+					  NULL,
+					  (GDestroyNotify)itdb_artwork_free);
+
+    for (gl=*ctx->artwork; gl; gl=gl->next)
+    {
+	Itdb_Track *track;
+	Itdb_Artwork *artwork=gl->data;
+	g_return_val_if_fail (artwork, -1);
+
+	g_hash_table_insert (mhii_id_hash, GINT_TO_POINTER (artwork->id), artwork);
+
+	/* add Artwork to track indicated by the dbid for backward
+	   compatibility */
+	track = get_song_by_dbid (itdb, artwork->dbid);
+	if (track == NULL)
+	{
+	    gchar *strval = g_strdup_printf("%" G_GINT64_FORMAT, artwork->dbid);
+	    g_print (_("Could not find corresponding track (dbid: %s) for artwork entry.\n"), strval);
+	    g_free (strval);
+	}
+	else
+	{
+	    if ((track->artwork_size + track->artwork_count) !=
+		artwork->artwork_size)
+	    {
+		g_warning (_("iTunesDB and ArtworkDB artwork sizes inconsistent (%d+%d != %d)\n"),
+			   track->artwork_size,
+			   track->artwork_count,
+			   track->artwork->artwork_size);
+	    }
+	    itdb_artwork_free (track->artwork);
+	    track->artwork = itdb_artwork_duplicate (artwork);
+	}
+    }
+    /* Now go through all the tracks and add artwork where an
+     * mhii_link is available */
+    for (gl=itdb->tracks; gl; gl=gl->next)
+    {
+	Itdb_Track *track = gl->data;
+	g_return_val_if_fail (track, -1);
+	if (track->mhii_link)
+	{
+	    Itdb_Artwork *artwork;
+	    artwork = g_hash_table_lookup (mhii_id_hash,
+					   GINT_TO_POINTER (track->mhii_link));
+	    if (artwork)
+	    {
+		g_return_val_if_fail (track->artwork, -1);
+		if (track->artwork->id != track->mhii_link)
+		{
+		    itdb_artwork_free (track->artwork);
+		    track->artwork = itdb_artwork_duplicate (artwork);
+		}
+		else
+		{
+		    /* same artwork -- don't copy again */
+		}
+	    }
+	    else
+	    {
+		/* The user can't do much here, so let's not worry them with
+		 * this warning...
+		gchar *strval = g_strdup_printf("%" G_GINT64_FORMAT, track->dbid);
+		g_print (_("Could not find artwork entry (mhii id: %u) for track (dbid: %s).\n"), track->mhii_link, strval);
+		g_free (strval);
+		 */
+
+		/* couldn't find artwork -- make sure track data is in
+		   a consistent state. */
+		itdb_track_remove_thumbnails (track);
+	    }
+	}
+    }
+    g_hash_table_destroy (mhii_id_hash);
+    /* The actual ItdbArtwork data was freed through the GHashTable
+       value_destroy_func */
+    g_list_free (*ctx->artwork);
+    ctx->artwork = NULL;
+
+    return 0;
+}
+	
+
+
 /* Database Object */
 static int
 parse_mhfd (DBParseContext *ctx, GError **error)
@@ -491,6 +594,8 @@ parse_mhfd (DBParseContext *ctx, GError **error)
 	DBParseContext *mhsd_context;
 	unsigned int cur_pos;
 	gint total_len;
+	gint32 i;
+	GList *artwork_glist = NULL;
 
 	mhfd = db_parse_context_get_m_header (ctx, MhfdHeader, "mhfd");
 	if (mhfd == NULL) {
@@ -503,38 +608,35 @@ parse_mhfd (DBParseContext *ctx, GError **error)
 	dump_mhfd (mhfd);
 	cur_pos = ctx->header_len;
 
-	/* The mhfd record always has 3 mhsd children, so it's hardcoded here.
-	 * It could be replaced with a loop using the nb_children field from
-	 * the mhfd record. [explanation by Christophe]
-	 */
-	mhsd_context = db_parse_context_get_sub_context (ctx, cur_pos);
-	if (mhsd_context == NULL) {
-		return -1;
+	if (ctx->db->db_type == DB_TYPE_ITUNES)
+	{   /* we need to collect all artwork in this GList for
+	       iTunesDB (see below) */
+	    ctx->artwork = &artwork_glist;
 	}
-	parse_mhsd (mhsd_context, NULL);
-	cur_pos += mhsd_context->total_len;
-	g_free (mhsd_context);
 
-	mhsd_context = db_parse_context_get_sub_context (ctx, cur_pos);
-	if (mhsd_context == NULL) {
+	for (i=0; i<mhfd->num_children; ++i)
+	{
+	    /* so far all mhfd we know have 3 children, but better be
+	       safe than sorry */
+	    mhsd_context = db_parse_context_get_sub_context (ctx, cur_pos);
+	    if (mhsd_context == NULL) {
 		return -1;
+	    }
+	    parse_mhsd (mhsd_context, NULL);
+	    cur_pos += mhsd_context->total_len;
+	    g_free (mhsd_context);
 	}
-	parse_mhsd (mhsd_context, NULL);
-	cur_pos += mhsd_context->total_len;
-	g_free (mhsd_context);
 
-	mhsd_context = db_parse_context_get_sub_context (ctx, cur_pos);
-	if (mhsd_context == NULL) {
-		return -1;
+	if (ctx->db->db_type == DB_TYPE_ITUNES)
+	{
+	    return mhfd_associate_itunesdb_artwork (ctx);
 	}
-	parse_mhsd (mhsd_context, NULL);
-	cur_pos += mhsd_context->total_len;
-	g_free (mhsd_context);
-	
+
 	return 0;
 }
 
 
+
 G_GNUC_INTERNAL char *
 ipod_db_get_artwork_db_path (const char *mount_point)
 {
@@ -575,83 +677,6 @@ ipod_db_get_artwork_db_path (const char *mount_point)
 	return filename;
 }
 
-G_GNUC_INTERNAL gboolean
-ipod_supports_cover_art (Itdb_Device *device)
-{
-	const Itdb_ArtworkFormat *formats;
-
-	if (device == NULL) {
-		return FALSE;
-	}
-
-	formats = itdb_device_get_artwork_formats (device);
-	if (formats == NULL) {
-		return FALSE;
-	}
-	
-	while (formats->type != -1)
-	{
-	    switch (formats->type)
-	    {
-	    case ITDB_THUMB_COVER_SMALL:
-	    case ITDB_THUMB_COVER_LARGE:
-		return TRUE;
-	    case ITDB_THUMB_PHOTO_SMALL:
-	    case ITDB_THUMB_PHOTO_LARGE:
-	    case ITDB_THUMB_PHOTO_FULL_SCREEN:
-	    case ITDB_THUMB_PHOTO_TV_SCREEN:
-		break;
-	    case ITDB_THUMB_COVER_XLARGE:
-	    case ITDB_THUMB_COVER_MEDIUM:
-	    case ITDB_THUMB_COVER_SMEDIUM:
-	    case ITDB_THUMB_COVER_XSMALL:
-		break;
-	    }
-	    formats++;
-	}
-
-	return FALSE;
-}
-
-G_GNUC_INTERNAL gboolean
-ipod_supports_photos (Itdb_Device *device)
-{
-	const Itdb_ArtworkFormat *formats;
-
-	if (device == NULL) {
-		return FALSE;
-	}
-
-	formats = itdb_device_get_artwork_formats (device);
-	if (formats == NULL) {
-		return FALSE;
-	}
-	
-	while (formats->type != -1)
-	{
-	    switch (formats->type)
-	    {
-	    case ITDB_THUMB_COVER_SMALL:
-	    case ITDB_THUMB_COVER_LARGE:
-		break;
-	    case ITDB_THUMB_PHOTO_SMALL:
-	    case ITDB_THUMB_PHOTO_LARGE:
-	    case ITDB_THUMB_PHOTO_FULL_SCREEN:
-	    case ITDB_THUMB_PHOTO_TV_SCREEN:
-		return TRUE;
-	    case ITDB_THUMB_COVER_XLARGE:
-	    case ITDB_THUMB_COVER_MEDIUM:
-	    case ITDB_THUMB_COVER_SMEDIUM:
-	    case ITDB_THUMB_COVER_XSMALL:
-		break;
-	    }
-	    formats++;
-	}
-
-	return FALSE;
-}
-
-
 int
 ipod_parse_artwork_db (Itdb_iTunesDB *itdb)
 {
@@ -664,7 +689,7 @@ ipod_parse_artwork_db (Itdb_iTunesDB *itdb)
 
 	g_return_val_if_fail (itdb, -1);
 
-	if (!ipod_supports_cover_art (itdb->device)) {
+	if (!itdb_device_supports_artwork (itdb->device)) {
 		return -1;
 	}
 	ctx = NULL;
@@ -679,17 +704,18 @@ ipod_parse_artwork_db (Itdb_iTunesDB *itdb)
 
 	ctx = db_parse_context_new_from_file (filename, &db);
 	g_free (filename);
+
 	if (ctx == NULL) {
 		goto error;
 	}	
 
 	parse_mhfd (ctx, NULL);
-	db_parse_context_destroy (ctx, TRUE);
+	db_parse_context_destroy (ctx);
 	return 0;
 
  error:
 	if (ctx != NULL) {
-		db_parse_context_destroy (ctx, TRUE);
+		db_parse_context_destroy (ctx);
 	}
 	return -1;
 }
@@ -757,7 +783,7 @@ ipod_parse_photo_db (Itdb_PhotoDB *photodb)
 		return  -1;
 	}
 	parse_mhfd (ctx, NULL);
-	db_parse_context_destroy (ctx, TRUE);
+	db_parse_context_destroy (ctx);
 
 	/* Now we need to replace references to artwork_ids in the
 	 * photo albums with references to the actual artwork
