@@ -20,7 +20,7 @@
  * 
  *  This product is not supported/written/published by Apple!
  *
- * $Id: db-itunes-parser.h 1729 2007-10-21 10:52:46Z teuf $
+ * $Id: db-itunes-parser.h 2040 2008-07-06 14:04:33Z teuf $
  */
 
 
@@ -28,9 +28,6 @@
 #define DB_PARSER_H
 
 #include <glib.h>
-/*#include "ipod-db-parser.h"*/
-
-#define ITUNESDB_MAX_SIZE 10 * 1024 * 1024
 
 struct _MHeader {
 	unsigned char header_id[4];
@@ -71,10 +68,8 @@ typedef struct _MhiaHeader MhiaHeader;
 
 typedef struct _MhitHeader471 MhitHeader471;
 /* MHOD typedef mess */
-typedef struct _ArtworkDB_MhodHeaderArtworkType3 ArtworkDB_MhodHeaderArtworkType3;
+typedef struct _ArtworkDB_MhodHeaderString ArtworkDB_MhodHeaderString;
 typedef struct _MhodHeaderString MhodHeaderString;
-typedef struct _MhodHeaderArtworkType1 MhodHeaderArtworkType1;
-/* typedef struct _MhodHeaderArtworkType3 MhodHeaderArtworkType3; */
 typedef struct _MhodHeaderSmartPlaylistData MhodHeaderSmartPlaylistData;
 typedef struct _MhodHeaderSmartPlaylistRuleString MhodHeaderSmartPlaylistRuleString; 
 typedef struct _MhodHeaderSmartPlaylistRuleNonString MhodHeaderSmartPlaylistRuleNonString;
@@ -158,7 +153,7 @@ struct _MhipHeader {
 	gint32 header_len;
 	gint32 total_len;
 	gint32 unknown1;
-	gint32 correlation_id;
+	gint32 format_id;
 	gint32 unknown2;
 	gint32 track_id;
 	gint32 timestamp;
@@ -362,57 +357,23 @@ enum MhodArtworkType {
 	MHOD_ARTWORK_TYPE_IMAGE      = 5  /* container: full resolution image (in the Photo Database) */
 };
 
-struct _MhodHeaderArtworkType1 {
+struct _ArtworkDB_MhodHeaderString {
 	unsigned char header_id[4];
 	gint32 header_len;
 	gint32 total_len;
-	gint16 type; 
-	gint8 unknown;
-	gint8 padding;
-	gint32 unknown1;
-	gint32 unknown2;
-	gint32 string_len;
-	gint32 unknown3; /* might be the string encoding */
-	gint32 unknown4; /* always zero? */
-	unsigned char string[];
-};
-
-#if 0
-/* not used at all */
-struct _MhodHeaderArtworkType3 {
-	unsigned char header_id[4];
-	gint32 header_len;
-	gint32 total_len;
-	gint16 type; /* 3 */
-	gint16 padding; /* high byte is padding length (0-3) */
-	gint32 unknown1;
-	gint32 unknown2;
-	gint32 string_len;
-	gint32 mhod_version;
-	gint32 unknown4;
-	gunichar2 string[];
-};
-#endif
-
-struct _ArtworkDB_MhodHeaderArtworkType3 {
-	unsigned char header_id[4];
-	gint32 header_len;
-	gint32 total_len;
-        gint16 type; /* 3 */
+        gint16 type;
         gint8  unknown13;
-	gint8  padding;
+	gint8  padding_len;
 	gint32 unknown1;
 	gint32 unknown2;
 	gint32 string_len;
-	gint8 mhod_version; /* 0,1: string is UTF8, 2: string is
-			       UTF16-LE */
+	gint8 encoding; /* 0,1: string is UTF8, 2: string is UTF16-LE */
         gint8 unknown5;
         gint16 unknown6;
 	gint32 unknown4;
         gchar  string[];
 };
 
-
 enum MhodLimitType {
 	MHOD_LIMIT_MINUTES = 1,
 	MHOD_LIMIT_MEGABYTES = 2,
@@ -598,7 +559,7 @@ struct _MhniHeader {
 	gint32 header_len;
 	gint32 total_len;
 	gint32 num_children;
-	gint32 correlation_id;
+	gint32 format_id;
 	gint32 ithmb_offset;
 	gint32 image_size;
 	gint16 vertical_padding;
@@ -657,7 +618,7 @@ struct _MhifHeader {
 	gint32 header_len;
 	gint32 total_len;
 	gint32 unknown1;
-	gint32 correlation_id;
+	gint32 format_id;
 	gint32 image_size;
 	unsigned char padding[];
 };
