@@ -23,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_private.h 1439 2007-05-27 14:58:41Z jcsjcs $
+|  $Id: itdb_private.h 1971 2008-05-11 07:17:29Z jcsjcs $
 */
 
 #ifndef __ITDB_PRIVATE_H__
@@ -32,7 +32,6 @@
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
-#include "glib-compat.h"
 #include "itdb_device.h"
 #include "itdb.h"
 
@@ -147,7 +146,7 @@ G_GNUC_INTERNAL void itdb_splr_free (Itdb_SPLRule *splr);
 G_GNUC_INTERNAL const gchar *itdb_photodb_get_mountpoint (Itdb_PhotoDB *photodb);
 G_GNUC_INTERNAL gchar *db_get_mountpoint (Itdb_DB *db);
 G_GNUC_INTERNAL Itdb_Device *db_get_device(Itdb_DB *db);
-G_GNUC_INTERNAL gint itdb_get_free_photo_id ( Itdb_PhotoDB *db );
+G_GNUC_INTERNAL gint itdb_get_max_photo_id ( Itdb_PhotoDB *db );
 G_GNUC_INTERNAL Itdb_iTunesDB *db_get_itunesdb (Itdb_DB *db);
 G_GNUC_INTERNAL Itdb_PhotoDB *db_get_photodb (Itdb_DB *db);
 G_GNUC_INTERNAL gint itdb_thumb_get_byteorder (ItdbThumbFormat format);
@@ -156,4 +155,8 @@ G_GNUC_INTERNAL time_t device_time_mac_to_time_t (Itdb_Device *device,
 G_GNUC_INTERNAL guint64 device_time_time_t_to_mac (Itdb_Device *device,
 						 time_t timet);
 G_GNUC_INTERNAL gint itdb_musicdirs_number_by_mountpoint (const gchar *mountpoint);
+G_GNUC_INTERNAL gboolean itdb_device_requires_checksum (Itdb_Device *device);
+G_GNUC_INTERNAL gboolean itdb_file_set_contents (const char *filename, 
+                                                 const char *data, gssize len, 
+                                                 GError **error);
 #endif
